function [gidx,gnames,glevels] = grp2idx(s)
//   Create index vector from a grouping variable.
//   Calling Sequence
//   [G,GN] = grp2idx(S)
//   [G,GN,GL] = grp2idx(S)
//   Parameters
//   S: grouping variable, can be a numeric, or logical vector; a cell vector of  strings; or a character matrix with each row representing a group label.
//   G: index vector taking integer values from 1 up to the number K  of distinct groups
//   GN: cell array of strings representing group  labels. GN(G) reproduces S 
//   GL: same as GN, but GL  has the same type as S
// Description
//
//   grp2idx treats NaNs or  empty strings  in S as missing   values and returns NaNs in the corresponding rows of G. 
//   GN and GL don't include entries for missing values.
//
//   See also
//   gscatter
//    grpstats
//   Examples
//     groups={'30s','40s','30s','40s','40s'}'
//     [G,GN]=grp2idx(groups)
//     disp(GN(G))

[nargout,nargin]=argn(0);

if ~isvector(s) 
    error( 'Grouping variable must be a vector or a character array or a call of strings.');
end
s = s(:);

if or(type(s)==[1 5 8])
    [glevels,dum,gidx] = unique2(s,'first');
    
    // Handle NaN missing values: return NaN group indices
    if ~isempty(glevels) & isnan(glevels($)) // NaNs are sorted to end
        glevels = glevels(~isnan(glevels));
        gidx(gidx > length(glevels)) = %nan;
    end
    if nargout > 1
        gnames = string(full(glevels));
    end

elseif type(s)==4
    [glevels,dum,gidx] = unique2(bool2s(s),'first');
    
    // Handle NaN missing values: return NaN group indices
    if ~isempty(glevels) & isnan(glevels($)) // NaNs are sorted to end
        glevels = glevels(~isnan(glevels));
        gidx(gidx > length(glevels)) = %nan;
    end
    if nargout > 1
        glevels=(glevels)==1
        gnames = string(glevels);
    end
    
elseif type(s)==10 

    [glevels,ord,gidx] = unique2(s,'first');

    [dum,reord] = gsort(ord,'g','i');
    ireord(reord) = 1:length(reord); ireord = ireord(:);
    
    // Handle empty string missing values: return NaN group indices
    if ~isempty(glevels) & (''==glevels(1)) // '' is sorted to beginning
        reord(reord==1) = [];
        ireord = ireord - (ireord > ireord(1));
        ireord(1) = %nan;
    end
    
    // Put the levels back into "first seen" order
    gidx = ireord(gidx(:)); // force a col, even for 0x0
    if nargout > 1
        glevels = glevels(reord(:)); // force a col, even for 0x0
        gnames = glevels;
    end
 
 elseif iscell(s)
   
    tmp=string(ones(size(s,1),1));
    for i=1:size(s,1)
       tmp(i)=s.entries(i);
    end
    s=tmp;
    [glevels,ord,gidx] = unique2(s,'first');

    [dum,reord] = gsort(ord,'g','i');
    ireord(reord) = 1:length(reord); ireord = ireord(:);
    
    // Handle empty string missing values: return NaN group indices
    if ~isempty(glevels) & (''==glevels(1)) // '' is sorted to beginning
        reord(reord==1) = [];
        ireord = ireord - (ireord > ireord(1));
        ireord(1) = %nan;
    end
    
    // Put the levels back into "first seen" order
    gidx = ireord(gidx(:)); // force a col, even for 0x0
    if nargout > 1
        glevels = glevels(reord(:)); // force a col, even for 0x0
        gnames = glevels;
        glevels=cellstr(gnames);
    end
    
else
    error('A grouping variable must be a categorical, numeric, or logical vector, a cell vector of strings, or a 2D character array.');
end
endfunction

function  [y, i, j] = unique2 (x,param)
 [nargout,nargin]=argn(0);
if (nargin < 1)
    error ("One Argument is needed!");
  end
if nargin==1
  optfirst = 0;
  optrows = 0;
else
      optfirst =  (param=="first");
      optlast = param=="last"
      optrows = param=="rows" & size (x, 2) > 1;

end
  if (optrows)
    n = size (x, 1);
    dim = 1;
  else
    n = size (x,'*');
    dim = (size (x, 1) == 1) + 1;
  end

  y = x;
  if (n < 1)
    i = [];
    j = [];
    return;
  elseif (n < 2)
    i = 1; 
    j = 1;
    return;
  end

  if (optrows)
    [y, i] =  gsort(y,'r','i');
    match = and (y(1:n-1,:) == y(2:n,:), 2);
    idx = find (match);
    y(idx,:) = [];
  else
    if (size (y, 1) ~= 1)
      y = y(:);
    end
    //[y, i] = mtlb_sort (y);
    [y, i] = gsort (y,'g','i')
    match = (y(1:n-1) == y(2:n));
    idx = find (match);
    y(idx) = [];
  end

  if (nargout >= 3)
    j = i;
    if (dim == 1)
      j(i) = cumsum ([1; ~match]);
    else
      j(i) = cumsum ([1, ~match]);
    end
  end

  if (optfirst)
    i(idx+1) = [];
  else
    i(idx) = [];
  end
endfunction